@@ -2,7 +2,9 @@
 
 """test_singledispatch_shim.py -- test code for ts2python's singledispatch"""
 
+from __future__ import annotations
 
+# from functools import singledispatch, singledispatchmethod
 from typing import List, Union
 from ts2python.singledispatch_shim import singledispatch, singledispatchmethod
 
@@ -35,16 +37,37 @@ class TestSingleDispatchShim:
             pass
 
 
+class A:
+    @singledispatchmethod
+    def func(self, param):
+        pass
+
+    @func.register
+    def _(self, param: C, a: int):
+        return a
+    @func.register
+    def _(self, b: complex, c: float):
+        return b, c
+
+
+
+class C:
+    pass
+
+
+# @A.func.register
+# def _(self, param: C, a: int):
+#     return a
+# @A.func.register
+# def _(self, b: complex, c: float):
+#     return b, c
+
+
 class TestForwardReference:
     def test_forward_reference(self):
-        @singledispatch
-        def func(param):
-            pass
-        @func.register
-        def _(param: 'C'):
-            pass
-        class C:
-            pass
+        a = A()
+        assert a.func(C(), 3) == 3
+        assert a.func((3 + 2j), 5.0) == ((3 + 2j), 5.0)
 
 
 class TestGenericAlias:
